﻿Class Window1 

#Region "Variables"
    ' The operator we are using.
    Private Enum Operators
        opNone
        opTimes
        opDivide
        opPlus
        opMinus
    End Enum
    Private m_Operator As Operators = Operators.opNone

    ' The first operand.
    Private m_SavedValue As Double = 0

    ' True when we need to start a new value.
    ' This is set when we click an operator.
    Private m_NewValue As Boolean = True

    ' True if we press C (used to know if we pressed CC).
    Private m_JustCleared As Boolean = False
#End Region ' Variables

#Region "Buttons"
    ' The user clicked a digit.
    Private Sub btnDigit_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnNum0.Click, btnNum1.Click, btnNum2.Click, btnNum3.Click, btnNum4.Click, btnNum5.Click, btnNum6.Click, btnNum7.Click, btnNum8.Click, btnNum9.Click
        Dim btn As Button = DirectCast(sender, Button)
        ProcessDigit(btn.Content.ToString())
    End Sub

    ' The user clicked the decimal point button.
    Private Sub btnDecimal_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnDecimal.Click
        ProcessDecimal()
    End Sub

    ' The user clicked CE.
    ' Clear just the current entry.
    Private Sub ClickCE(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs)
        ProcessCE()
    End Sub

    ' The user clicked C.
    ' Clear the current value, saved value, and operator.
    Private Sub btnC_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnC.Click
        ProcessC()
    End Sub

    ' Calculate a result.
    Private Sub btnEquals_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnEquals.Click
        ProcessEquals()
    End Sub

    ' Store an operator other than equals.
    Private Sub btnOperator_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnPlus.Click, btnMinus.Click, btnTimes.Click, btnDivide.Click
        ' Process the operator.
        Dim btn As Button = DirectCast(sender, Button)
        Select Case btn.Content.ToString()
            Case "+"
                ProcessOperator(Operators.opPlus)
            Case "-"
                ProcessOperator(Operators.opMinus)
            Case "*"
                ProcessOperator(Operators.opTimes)
            Case "/"
                ProcessOperator(Operators.opDivide)
        End Select
    End Sub

    ' The user clicked "+/-".
    Private Sub btnPlusMinus_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnPlusMinus.Click
        ProcessPlusMinus()
    End Sub
#End Region ' Buttons

#Region "Keys"
    ' Process Delete, Backspace, and Enter.
    Private Sub Window1_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Input.KeyEventArgs) Handles Me.KeyDown
        Select Case e.Key
            Case Key.Delete, Key.Back
                DeleteCharacter()
            Case Key.Enter
                ProcessEquals()
        End Select
    End Sub

    ' Process digits, decimal point, and operators.
    Private Sub Window1_TextInput(ByVal sender As Object, ByVal e As System.Windows.Input.TextCompositionEventArgs) Handles Me.TextInput
        Select Case e.Text
            Case "0" To "9"         ' Digits.
                ProcessDigit(e.Text)

            Case "."                ' Decimal point.
                ProcessDecimal()

            Case "C", "c"           ' C.
                If m_JustCleared Then
                    ' Clear all.
                    ProcessC()
                Else
                    ' Clear entry.
                    ProcessCE()
                End If

            Case "+"                ' Operators.
                ProcessOperator(Operators.opPlus)
            Case "-"
                ProcessOperator(Operators.opMinus)
            Case "*"
                ProcessOperator(Operators.opTimes)
            Case "/"
                ProcessOperator(Operators.opDivide)

            Case "="
                ProcessOperator(Operators.opNone)
        End Select
    End Sub
#End Region ' Keys

#Region "Process User Actions"
    ' Remove the last character.
    Private Sub DeleteCharacter()
        ' Remove the sign.
        Dim txt As String = lblResult.Content.ToString()
        Dim sign As String = ""
        If txt.StartsWith("-") Then
            sign = "-"
            txt = txt.Substring(1)
        End If

        ' Remove the final digit.
        txt = txt.Substring(0, txt.Length - 1)

        ' Replace the sign and clean it up.
        txt = sign & txt
        lblResult.Content = CleanResult(txt)

        m_JustCleared = False
    End Sub

    ' Add a digit to the output.
    Private Sub ProcessDigit(ByVal digit As String)
        ' If this is a new value, start with nothing.
        Dim txt As String = ""
        If Not m_NewValue Then
            txt = lblResult.Content.ToString()
        End If

        ' Add the digit and clean it up.
        lblResult.Content = CleanResult(txt & digit)

        m_JustCleared = False
        m_NewValue = False
    End Sub

    ' Add a decimal point to the output.
    Private Sub ProcessDecimal()
        Dim txt As String = lblResult.Content.ToString()
        If txt.Contains(",") Then
            Beep()
        Else
            ProcessDigit(",")
        End If
    End Sub

    ' Remove leading zeros.
    Private Function CleanResult(ByVal txt As String) As String
        ' Remove the sign.
        Dim sign As String = ""
        If txt.StartsWith("-") Then
            txt = txt.Substring(1)
            sign = "-"
        End If

        ' Remove leading zeros.
        Do While txt.StartsWith("0")
            txt = txt.Substring(1)
        Loop

        ' Add leading zero if it starts with
        ' a decimal point or has zero length.
        If txt.StartsWith(",") OrElse _
            txt.Length = 0 _
        Then
            txt = "0" & txt
        End If

        ' Add the sign and return.
        Return sign & txt
    End Function

    ' The user clicked =.
    Private Sub ProcessEquals()
        Dim txt As String = lblResult.Content.ToString()
        Dim new_value As Double = Double.Parse(txt)

        Select Case m_Operator
            Case Operators.opNone
                m_SavedValue = new_value
            Case Operators.opPlus
                m_SavedValue = m_SavedValue + new_value
            Case Operators.opMinus
                m_SavedValue = m_SavedValue - new_value
            Case Operators.opTimes
                m_SavedValue = m_SavedValue * new_value
            Case Operators.opDivide
                m_SavedValue = m_SavedValue / new_value
        End Select

        m_Operator = Operators.opNone

        ' Display the result.
        lblResult.Content = m_SavedValue.ToString

        ' Strobe the result to provide visual feedback.
        StrobeResult()

        ' Start a new value.
        m_NewValue = True

        ' We did not just clear.
        m_JustCleared = False
    End Sub

    ' Process an operator.
    Private Sub ProcessOperator(ByVal op As Operators)
        ' Perform the previous operation.
        ' This also:
        '   Strobes the result
        '   Sets m_NewValue = True
        '   Sets m_JustCleared = False
        ProcessEquals()

        ' Remember this operation.
        m_Operator = op
    End Sub

    Private Sub ProcessPlusMinus()
        Dim txt As String = lblResult.Content.ToString()
        If txt.StartsWith("-") Then
            txt = txt.Substring(1)
        Else
            txt = "-" & txt
        End If

        lblResult.Content = txt

        m_NewValue = False
        m_JustCleared = False
    End Sub

    ' The user clicked CE.
    Private Sub ProcessCE()
        lblResult.Content = "0"
        m_NewValue = True
    End Sub

    ' The user clicked C.
    Private Sub ProcessC()
        m_SavedValue = 0
        m_Operator = Operators.opNone

        ProcessCE()
    End Sub
#End Region ' Process User Actions

    ' Make the result fade away and then
    ' come back to give some visual feedback.
    Private Sub StrobeResult()
        ' Make the DoubleAnimation.
        Dim opacity_a As New Animation.DoubleAnimation()
        opacity_a.From = 1.0
        opacity_a.To = 0.0
        opacity_a.Duration = _
            New Duration(TimeSpan.FromSeconds(0.2))
        opacity_a.FillBehavior = Animation.FillBehavior.Stop
        ' opacity_a.AutoReverse = True

        ' We are animating the lblResult
        ' Control's Opacity property.
        Animation.Storyboard.SetTargetName( _
            opacity_a, lblResult.Name)
        Animation.Storyboard.SetTargetProperty( _
            opacity_a, New PropertyPath(Button.OpacityProperty))

        ' Add the animation to a StoryBoard.
        Dim sb As New Animation.Storyboard()
        sb.Children.Add(opacity_a)

        ' Begin the StoryBoard.
        sb.Begin(Me)

        ' Play a click sound.
        My.Computer.Audio.Play( _
            My.Resources.Windows_Navigation_Start, _
            AudioPlayMode.Background)
    End Sub
End Class
